package org.artifactory.metrics;

import org.apache.commons.lang.StringUtils;
import org.apache.http.HttpHeaders;
import org.artifactory.api.repo.RepositoryService;
import org.artifactory.metrics.callhome.CallHomeDataModel;
import org.artifactory.metrics.callhome.CallHomeMetrics;
import org.artifactory.util.HttpUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;

@Service
public class MetricsListenerService {
    RepositoryService repositoryService;
    CallHomeMetrics callHomeMetrics;

    @Autowired
    public MetricsListenerService(RepositoryService repositoryService, CallHomeMetrics callHomeMetrics) {
        this.repositoryService = repositoryService;
        this.callHomeMetrics = callHomeMetrics;
    }

    public void send(int statusCode, long responseContentLength, HttpServletRequest request) {
        String repoName = MetricsThreadLocal.getRepoName();
        if (StringUtils.isNotBlank(repoName)) {
            String action = MetricsThreadLocal.getAction();
            String packageType = repositoryService.repoDescriptorByKey(repoName).getType()
                    .getDisplayName();
            String userAgent = request.getHeader(HttpHeaders.USER_AGENT);
            Long requestContentLength = HttpUtils.getContentLength(request);
            String remoteAddress = HttpUtils.getRemoteClientAddress(request);
            String SERVICE = "ARTIFACTORY";
            CallHomeDataModel metricsObject = CallHomeDataModel.builder()
                    .service(SERVICE)
                    .action(action)
                    .statusCode(statusCode)
                    .tool(StringUtils.substringBefore(StringUtils.substringBefore(userAgent, "/"), " "))
                    .remoteAddress(remoteAddress)
                    .packageType(packageType)
                    .userAgent(userAgent)
                    .requestContentLength(requestContentLength)
                    .responseContentLength(responseContentLength)
                    .build();
            callHomeMetrics.send(metricsObject);
        }
    }
}
