package org.artifactory.storage.db.upgrades.itest.version;

import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsAndTmpAndErrorsDBSqlConverter;
import org.artifactory.storage.db.itest.DbTestUtils;
import org.artifactory.storage.db.upgrades.common.UpgradeBaseTest;
import org.artifactory.storage.db.version.ArtifactoryDBVersion;
import org.artifactory.storage.db.version.converter.DBConverter;
import org.jfrog.storage.DbType;
import org.jfrog.storage.util.DbUtils;
import org.testng.annotations.Test;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;

/**
 * Test SQLs of @{@link V227NodeEventsAndTmpAndErrorsDBSqlConverter}
 *
 * @author shaharz
 */
@SuppressWarnings("SameParameterValue")
@Test
public class V227UpgradeNodeEventsAndTmpAndErrorsITest extends UpgradeBaseTest {

    /**
     * Verify Upgrade
     */
    public void testUpgrade() throws Exception {
        // Go back to old version and run upgrade
        resetToVersion(ArtifactoryDBVersion.v226);

        // Verify revert succeed
        verify(false);

        for (DBConverter dbConverter : ArtifactoryDBVersion.v227.getConverters()) {
            dbConverter.convert(jdbcHelper, dbProperties.getDbType());
        }

        // Verify upgrade
        verify(true);
    }

    private void verify(boolean value) throws SQLException {
        assertEquals(tableExists("node_events_tmp"), value);
        assertEquals(columnExists("node_events_tmp", "tmp_event_id"), value);
        assertEquals(columnExists("node_events_tmp", "timestamp"), value);
        assertEquals(columnExists("node_events_tmp", "event_type"), value);
        assertEquals(columnExists("node_events_tmp", "path"), value);
        assertEquals(columnExists("node_events_tmp", "hints"), value);
        if (!dbProperties.getDbType().equals(DbType.DERBY)) {
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_tmp", "timestamp",
                    "node_events_tmp_idx", dbProperties.getDbType()), value);
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_tmp", "tmp_event_id",
                    "node_events_tmp_idx", dbProperties.getDbType()), value);
        }

        assertEquals(columnExists("node_events", "hints"), value);
        try {
            assertEquals(isColumnNullable("node_events", "hints"), value);
        } catch (Exception e) {
            assertFalse(value, e.getMessage());
        }
        assertEquals(columnExists("node_events_errors", "event_id"), value);
        try {
            assertEquals(isColumnNullable("node_events_errors", "event_id"), value);
        } catch (Exception e) {
            assertFalse(value, e.getMessage());
        }
        assertEquals(DbUtils.foreignKeyExists(jdbcHelper, dbProperties.getDbType(), "node_events_errors",
                "node_events_errors_event_id_fk"), value);

        if (!dbProperties.getDbType().equals(DbType.DERBY)) {
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_errors", "operation_key",
                    "node_events_errors_op_key_idx", dbProperties.getDbType()), !value);
        }

        if (!dbProperties.getDbType().equals(DbType.DERBY) || value) {
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_errors", "operation_key",
                    "node_events_errors_idx", dbProperties.getDbType()), value);
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_errors", "event_category",
                    "node_events_errors_idx", dbProperties.getDbType()), value);
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_errors", "event_id",
                    "node_events_errors_idx", dbProperties.getDbType()), value);
            assertEquals(DbUtils.indexExists(jdbcHelper, "node_events_errors", "event_time",
                    "node_events_errors_idx", dbProperties.getDbType()), value);
        }

        assertEquals(isSequenceExistOrAutoIncrement(), value);
    }

    private boolean isSequenceExistOrAutoIncrement() throws SQLException {
        if (dbProperties.getDbType().equals(DbType.MYSQL) || dbProperties.getDbType().equals(DbType.MARIADB)) {
            try (ResultSet resultSet = jdbcHelper.executeSelect("SELECT event_id FROM node_events")) {
                ResultSetMetaData metadata = resultSet.getMetaData();
                return metadata.isAutoIncrement(1);
            }
        } else {
            return DbUtils.withConnection(jdbcHelper, conn ->
                    DbTestUtils.sequenceExists(conn, dbProperties.getDbType(), "node_events_event_id_seq"));

        }
    }
}
