package org.artifactory.ui.rest.service.admin.configuration.repositories;

import org.artifactory.api.build.BuildService;
import org.artifactory.api.config.CentralConfigService;
import org.artifactory.pipelines.InternalPipelinesService;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.ArtifactoryRestResponse;
import org.testng.Assert;
import org.testng.annotations.Test;

import static org.mockito.Mockito.anyInt;
import static org.mockito.Mockito.anyString;
import static org.mockito.Mockito.doAnswer;
import static org.mockito.Mockito.mock;
import static org.mockito.Mockito.when;

/**
 * @author Alexei Vainshtein
 */
public class DeleteRepositoryConfigServiceTest {

    @Test
    public void testExecuteDeletePipeInfoRepo() {
        BuildService buildService = mock(BuildService.class);
        InternalPipelinesService pipelinesService = mock(InternalPipelinesService.class);
        CentralConfigService centralConfigService = mock(CentralConfigService.class);
        DeleteRepositoryConfigService<?> deleteRepositoryConfigService =
                new DeleteRepositoryConfigService<>(centralConfigService, null, null, buildService, null,
                        pipelinesService);
        ArtifactoryRestRequest<?> request = mock(ArtifactoryRestRequest.class);
        when(request.getPathParamByKey("repoKey")).thenReturn("pipe-info");
        when(pipelinesService.isPipeInfoRepo("pipe-info")).thenReturn(true);
        ArtifactoryRestResponse<?> restResponse = mock(ArtifactoryRestResponse.class);
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            Object argument = invocationOnMock.getArguments()[0];
            Assert.assertTrue(argument instanceof String);
            Assert.assertEquals(argument, "Deleting a system repo 'pipe-info' is not allowed.");
            return restResponse;
        }).when(restResponse).error(anyString());
        doAnswer(invocationOnMock -> {
            Assert.assertEquals(invocationOnMock.getArguments().length, 1);
            int argument = (int) invocationOnMock.getArguments()[0];
            Assert.assertEquals(argument, 403);
            return restResponse;
        }).when(restResponse).responseCode(anyInt());
        deleteRepositoryConfigService.execute(request, restResponse);
    }
}