/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.rest.resource.search.types;

import org.artifactory.addon.rest.RestAddon;
import org.artifactory.api.build.BuildService;
import org.artifactory.api.context.ContextHelper;
import org.artifactory.api.rest.constant.SearchRestConstants;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.rest.exception.BadRequestException;

import javax.annotation.Nullable;
import javax.annotation.security.RolesAllowed;
import javax.servlet.http.HttpServletRequest;
import javax.ws.rs.GET;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.io.IOException;

import static org.artifactory.projects.ProjectsService.PROJECT;
import static org.artifactory.rest.resource.ci.BuildResource.BUILD_REPO;

/**
 * @author Noam Y. Tenne
 */
@RolesAllowed({AuthorizationService.ROLE_USER, AuthorizationService.ROLE_ADMIN})
public class DependencySearchResource {

    private RestAddon restAddon;
    private HttpServletRequest request;

    public DependencySearchResource(RestAddon restAddon, HttpServletRequest request) {
        this.restAddon = restAddon;
        this.request = request;
    }

    @GET
    @Produces({SearchRestConstants.MT_DEPENDENCY_BUILDS, MediaType.APPLICATION_JSON})
    public Response get(
            @QueryParam(SearchRestConstants.PARAM_SHA1_CHECKSUM) String sha1,
            @QueryParam(SearchRestConstants.PARAM_SHA2_CHECKSUM) String sha2,
            @QueryParam(BUILD_REPO) @Nullable String buildRepo,
            @QueryParam(PROJECT) @Nullable String projectKey) throws IOException {
        try {
            BuildService buildService = ContextHelper.get().beanForType(BuildService.class);
            return restAddon.searchDependencyBuilds(request, sha1, sha2,
                    buildService.getPreferredBuildRepo(buildRepo, projectKey));
        } catch (IllegalArgumentException iae) {
            throw new BadRequestException(iae.getMessage());
        }
    }
}
