package org.artifactory.rest.services.artifact.lifecycle;

import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleAddon;
import org.artifactory.api.rest.artifact.lifecycle.ArtifactLifecycleRestoreRequest;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;

/**
 * @author Omri Ziv
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class ArtifactLifecycleRestoreService implements RestService<ArtifactLifecycleRestoreRequest> {

    private static final Logger log = LoggerFactory.getLogger(ArtifactLifecycleRestoreService.class);

    @Autowired
    private AddonsManager addonsManager;

    @Override
    public void execute(ArtifactoryRestRequest<ArtifactLifecycleRestoreRequest> request, RestResponse response) {
        ArtifactLifecycleRestoreRequest restoreRequest = request.getImodel();
        try {
            addonsManager.addonByType(ArtifactLifecycleAddon.class).restore(restoreRequest);
            response.info("Restore operation was triggered successfully.");
            response.responseCode(HttpServletResponse.SC_ACCEPTED);
        } catch (IOException e) {
            log.error("Failed to restore, caught exception: ", e);
            response.error("Failed to restore - " + e.getMessage());
        }
    }
}
