package org.artifactory.rest.services.artifact.lifecycle;

import org.apache.commons.collections.CollectionUtils;
import org.artifactory.addon.AddonsManager;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecycleAddon;
import org.artifactory.addon.artifact.lifecycle.ArtifactLifecyclePolicyModel;
import org.artifactory.rest.common.service.ArtifactoryRestRequest;
import org.artifactory.rest.common.service.RestResponse;
import org.artifactory.rest.common.service.RestService;
import org.jfrog.common.StreamSupportUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author Omri Ziv
 */
@Component
@Scope(BeanDefinition.SCOPE_PROTOTYPE)
public class DeleteArtifactLifecyclePoliciesService implements RestService<List<ArtifactLifecyclePolicyModel>> {

    @Autowired
    private AddonsManager addonsManager;

    @Override
    public void execute(ArtifactoryRestRequest<List<ArtifactLifecyclePolicyModel>> request, RestResponse response) {
        List<ArtifactLifecyclePolicyModel> policyModels = request.getImodel();
        ArtifactLifecycleAddon archivingAddon = addonsManager.addonByType(ArtifactLifecycleAddon.class);
        StreamSupportUtils.stream(policyModels)
                .map(ArtifactLifecyclePolicyModel::getKey)
                .forEach(archivingAddon::deletePolicy);
        response.info("Successfully delete " + CollectionUtils.size(policyModels) + "retention policies.");
    }
}
